@@ -1,3 +1,4 @@
+import os
 import time
 import grpc
 from typing import List, Optional, Tuple, Union
@@ -40,7 +41,7 @@ from .proto.exchange import (
     injective_explorer_rpc_pb2 as explorer_rpc_pb,
     injective_explorer_rpc_pb2_grpc as explorer_rpc_grpc,
     injective_auction_rpc_pb2 as auction_rpc_pb,
-    injective_auction_rpc_pb2_grpc as auction_rpc_grpc
+    injective_auction_rpc_pb2_grpc as auction_rpc_grpc,
 )
 
 from .constant import Network
@@ -52,47 +53,84 @@ class Client:
         insecure: bool = False,
         credentials: grpc.ChannelCredentials = None,
     ):
+        # load root CA cert
+        if not insecure:
+            if network.env == 'testnet':
+                if credentials is None:
+                    with open(os.path.join(os.path.dirname(__file__), 'cert/testnet.crt'), 'rb') as f:
+                        credentials = grpc.ssl_channel_credentials(f.read())
+            if network.env == 'mainnet':
+                if credentials is None:
+                    with open(os.path.join(os.path.dirname(__file__), 'cert/mainnet.crt'), 'rb') as f:
+                        credentials = grpc.ssl_channel_credentials(f.read())
+
         # chain stubs
-        chain_channel = (
+        self.chain_channel = (
             grpc.insecure_channel(network.grpc_endpoint)
-            if insecure
-            else grpc.secure_channel(
-                network.grpc_endpoint,
-                credentials or grpc.ssl_channel_credentials(),
-            )
+            if insecure else grpc.secure_channel(network.grpc_endpoint, credentials)
         )
-        self.stubCosmosTendermint = tendermint_query_grpc.ServiceStub(chain_channel)
-        self.stubAuth = auth_query_grpc.QueryStub(chain_channel)
-        self.stubAuthz = authz_query_grpc.QueryStub(chain_channel)
-        self.stubTx = tx_service_grpc.ServiceStub(chain_channel)
+        self.stubCosmosTendermint = tendermint_query_grpc.ServiceStub(self.chain_channel)
+        self.stubAuth = auth_query_grpc.QueryStub(self.chain_channel)
+        self.stubAuthz = authz_query_grpc.QueryStub(self.chain_channel)
+        self.stubTx = tx_service_grpc.ServiceStub(self.chain_channel)
+        self.chain_cookie = ""
+        self.exchange_cookie = ""
 
         # exchange stubs
-        exchange_channel = (
+        self.exchange_channel = (
             grpc.insecure_channel(network.grpc_exchange_endpoint)
-            if insecure
-            else grpc.secure_channel(
-                network.grpc_endpoint,
-                credentials or grpc.ssl_channel_credentials(),
-            )
+            if insecure else grpc.secure_channel(network.grpc_exchange_endpoint, credentials)
         )
-        self.stubMeta = exchange_meta_rpc_grpc.InjectiveMetaRPCStub(exchange_channel)
+
+        self.stubMeta = exchange_meta_rpc_grpc.InjectiveMetaRPCStub(self.exchange_channel)
         self.stubExchangeAccount = exchange_accounts_rpc_grpc.InjectiveAccountsRPCStub(
-            exchange_channel
+            self.exchange_channel
         )
-        self.stubOracle = oracle_rpc_grpc.InjectiveOracleRPCStub(exchange_channel)
+        self.stubOracle = oracle_rpc_grpc.InjectiveOracleRPCStub(self.exchange_channel)
         self.stubInsurance = insurance_rpc_grpc.InjectiveInsuranceRPCStub(
-            exchange_channel
+            self.exchange_channel
         )
         self.stubSpotExchange = spot_exchange_rpc_grpc.InjectiveSpotExchangeRPCStub(
-            exchange_channel
+            self.exchange_channel
         )
         self.stubDerivativeExchange = (
             derivative_exchange_rpc_grpc.InjectiveDerivativeExchangeRPCStub(
-                exchange_channel
+                self.exchange_channel
             )
         )
-        self.stubExplorer = explorer_rpc_grpc.InjectiveExplorerRPCStub(exchange_channel)
-        self.stubAuction = auction_rpc_grpc.InjectiveAuctionRPCStub(exchange_channel)
+        self.stubExplorer = explorer_rpc_grpc.InjectiveExplorerRPCStub(self.exchange_channel)
+        self.stubAuction = auction_rpc_grpc.InjectiveAuctionRPCStub(self.exchange_channel)
+
+    def close_exchange_channel(self):
+        self.exchange_channel.close()
+
+    def close_chain_channel(self):
+        self.chain_channel.close()
+
+    # cookie helper methods
+    def get_cookie(self, type):
+        metadata = None
+        if type == "chain":
+            if self.chain_cookie != "":
+                metadata = (("cookie", self.chain_cookie),)
+        if type == "exchange":
+            if self.exchange_cookie != "":
+                metadata = (("cookie", self.exchange_cookie),)
+        return metadata
+
+    def set_cookie(self, call, type):
+        new_cookie = None
+        for k, v in call.initial_metadata():
+            if k == "set-cookie":
+                new_cookie = v
+
+        if new_cookie == None:
+            return
+
+        if type == "chain":
+            self.chain_cookie = new_cookie
+        if type == "exchange":
+            self.exchange_cookie = new_cookie
 
     # default client methods
     def get_latest_block(self) -> tendermint_query.GetLatestBlockResponse:
@@ -135,33 +173,48 @@ class Client:
         self, tx_byte: bytes
     ) -> Tuple[Union[abci_type.SimulationResponse, grpc.RpcError], bool]:
         try:
-            return (
-                self.stubTx.Simulate(tx_service.SimulateRequest(tx_bytes=tx_byte)),
-                True,
-            )
+            req = tx_service.SimulateRequest(tx_bytes=tx_byte)
+
+            metadata = self.get_cookie(type="chain")
+            res, call = self.stubTx.Simulate.with_call(req, metadata=metadata)
+            self.set_cookie(call,type="chain")
+
+            return res, True
+
         except grpc.RpcError as err:
             return err, False
 
     def send_tx_sync_mode(self, tx_byte: bytes) -> abci_type.TxResponse:
-        return self.stubTx.BroadcastTx(
-            tx_service.BroadcastTxRequest(
-                tx_bytes=tx_byte, mode=tx_service.BroadcastMode.BROADCAST_MODE_SYNC
-            )
-        ).tx_response
+        req = tx_service.BroadcastTxRequest(
+            tx_bytes=tx_byte, mode=tx_service.BroadcastMode.BROADCAST_MODE_SYNC
+        )
+        metadata = self.get_cookie(type="chain")
+        res, call = self.stubTx.BroadcastTx.with_call(req, metadata=metadata)
+        self.set_cookie(call,type="chain")
+
+        return res.tx_response
 
     def send_tx_async_mode(self, tx_byte: bytes) -> abci_type.TxResponse:
-        return self.stubTx.BroadcastTx(
-            tx_service.BroadcastTxRequest(
-                tx_bytes=tx_byte, mode=tx_service.BroadcastMode.BROADCAST_MODE_ASYNC
-            )
-        ).tx_response
+        req = tx_service.BroadcastTxRequest(
+            tx_bytes=tx_byte,
+            mode=tx_service.BroadcastMode.BROADCAST_MODE_ASYNC
+        )
+        metadata = self.get_cookie(type="chain")
+        res, call = self.stubTx.BroadcastTx.with_call(req, metadata=metadata)
+        self.set_cookie(call,type="chain")
+
+        return res.tx_response
 
     def send_tx_block_mode(self, tx_byte: bytes) -> abci_type.TxResponse:
-        return self.stubTx.BroadcastTx(
-            tx_service.BroadcastTxRequest(
-                tx_bytes=tx_byte, mode=tx_service.BroadcastMode.BROADCAST_MODE_BLOCK
-            )
-        ).tx_response
+        req = tx_service.BroadcastTxRequest(
+            tx_bytes=tx_byte,
+            mode=tx_service.BroadcastMode.BROADCAST_MODE_BLOCK
+        )
+        metadata = self.get_cookie(type="chain")
+        res, call = self.stubTx.BroadcastTx.with_call(req, metadata=metadata)
+        self.set_cookie(call,type="chain")
+
+        return res.tx_response
 
     def get_chain_id(self) -> str:
         latest_block = self.get_latest_block()
@@ -170,7 +223,11 @@ class Client:
     def get_grants(self, granter: str, grantee: str, **kwargs):
         return self.stubAuthz.Grants(
             authz_query.QueryGrantsRequest(
-                granter=granter, grantee=grantee, msg_type_url=kwargs.get("msg_type_url")))
+                granter=granter,
+                grantee=grantee,
+                msg_type_url=kwargs.get("msg_type_url"),
+            )
+        )
 
     # Injective Exchange client methods
 
@@ -240,17 +297,15 @@ class Client:
         )
         return self.stubExchangeAccount.SubaccountBalancesList(req)
 
-    def get_subaccount_history(
-        self, subaccount_id: str, **kwargs
-    ):
+    def get_subaccount_history(self, subaccount_id: str, **kwargs):
         req = exchange_accounts_rpc_pb.SubaccountHistoryRequest(
-            subaccount_id=subaccount_id, denom=kwargs.get("denom"), transfer_types=kwargs.get("transfer_types")
+            subaccount_id=subaccount_id,
+            denom=kwargs.get("denom"),
+            transfer_types=kwargs.get("transfer_types"),
         )
         return self.stubExchangeAccount.SubaccountHistory(req)
 
-    def get_subaccount_order_summary(
-        self, subaccount_id: str, **kwargs
-    ):
+    def get_subaccount_order_summary(self, subaccount_id: str, **kwargs):
         req = exchange_accounts_rpc_pb.SubaccountOrderSummaryRequest(
             subaccount_id=subaccount_id,
             order_direction=kwargs.get("order_direction"),
@@ -258,9 +313,7 @@ class Client:
         )
         return self.stubExchangeAccount.SubaccountOrderSummary(req)
 
-    def get_order_states(
-        self, **kwargs
-    ):
+    def get_order_states(self, **kwargs):
         req = exchange_accounts_rpc_pb.OrderStatesRequest(
             spot_order_hashes=kwargs.get("spot_order_hashes"),
             derivative_order_hashes=kwargs.get("derivative_order_hashes"),
@@ -272,7 +325,9 @@ class Client:
         return self.stubExchangeAccount.Portfolio(req)
 
     def get_rewards(self, **kwargs):
-        req = exchange_accounts_rpc_pb.RewardsRequest(account_address=kwargs.get("account_address"), epoch=kwargs.get("epoch"))
+        req = exchange_accounts_rpc_pb.RewardsRequest(
+            account_address=kwargs.get("account_address"), epoch=kwargs.get("epoch")
+        )
         return self.stubExchangeAccount.Rewards(req)
 
     # OracleRPC
@@ -310,89 +365,103 @@ class Client:
         req = insurance_rpc_pb.FundsRequest()
         return self.stubInsurance.Funds(req)
 
-    def get_redemptions(
-        self, **kwargs
-    ):
+    def get_redemptions(self, **kwargs):
         req = insurance_rpc_pb.RedemptionsRequest(
-            redeemer=kwargs.get("redeemer"), redemption_denom=kwargs.get("redemption_denom"), status=kwargs.get("status")
+            redeemer=kwargs.get("redeemer"),
+            redemption_denom=kwargs.get("redemption_denom"),
+            status=kwargs.get("status"),
         )
         return self.stubInsurance.Redemptions(req)
 
-   # SpotRPC
+    # SpotRPC
 
     def get_spot_market(self, market_id: str):
         req = spot_exchange_rpc_pb.MarketRequest(market_id=market_id)
         return self.stubSpotExchange.Market(req)
 
-    def get_spot_markets(
-        self, **kwargs
-    ):
+    def get_spot_markets(self, **kwargs):
         req = spot_exchange_rpc_pb.MarketsRequest(
-            market_status=kwargs.get("market_status"), base_denom=kwargs.get("base_denom"), quote_denom=kwargs.get("quote_denom")
+            market_status=kwargs.get("market_status"),
+            base_denom=kwargs.get("base_denom"),
+            quote_denom=kwargs.get("quote_denom"),
         )
         return self.stubSpotExchange.Markets(req)
 
-    def stream_spot_markets(self):
-        req = spot_exchange_rpc_pb.StreamMarketsRequest()
-        return self.stubSpotExchange.StreamMarkets(req)
+    def stream_spot_markets(self, **kwargs):
+        req = spot_exchange_rpc_pb.StreamMarketsRequest(
+            market_ids=kwargs.get("market_ids"))
+        metadata = self.get_cookie(type="exchange")
+        res = self.stubSpotExchange.StreamMarkets(req, metadata=metadata)
+        self.set_cookie(res,type="exchange")
+        return res.tx_response
 
     def get_spot_orderbook(self, market_id: str):
         req = spot_exchange_rpc_pb.OrderbookRequest(market_id=market_id)
         return self.stubSpotExchange.Orderbook(req)
+    
+    def get_spot_orderbooks(self, market_ids: List):
+        req = spot_exchange_rpc_pb.OrderbooksRequest(market_ids=market_ids)
+        return self.stubSpotExchange.Orderbooks(req)
 
-    def get_spot_orders(
-        self, market_id: str, **kwargs
-    ):
+    def get_spot_orders(self, market_id: str, **kwargs):
         req = spot_exchange_rpc_pb.OrdersRequest(
-            market_id=market_id, order_side=kwargs.get("order_side"), subaccount_id=kwargs.get("subaccount_id")
+            market_id=market_id,
+            order_side=kwargs.get("order_side"),
+            subaccount_id=kwargs.get("subaccount_id"),
         )
         return self.stubSpotExchange.Orders(req)
 
-    def get_spot_trades(
-        self,
-        market_id: str,
-        **kwargs
-    ):
+    def get_spot_trades(self, market_id: str, **kwargs):
         req = spot_exchange_rpc_pb.TradesRequest(
             market_id=market_id,
             execution_side=kwargs.get("execution_side"),
             direction=kwargs.get("direction"),
             subaccount_id=kwargs.get("subaccount_id"),
             skip=kwargs.get("skip"),
-            limit=kwargs.get("limit")
+            limit=kwargs.get("limit"),
         )
         return self.stubSpotExchange.Trades(req)
 
     def stream_spot_orderbook(self, market_id: str):
         req = spot_exchange_rpc_pb.StreamOrderbookRequest(market_ids=[market_id])
-        return self.stubSpotExchange.StreamOrderbook(req)
+        metadata = self.get_cookie(type="exchange")
+        res = self.stubSpotExchange.StreamOrderbook(req, metadata=metadata)
+        self.set_cookie(res,type="exchange")
+        return res
 
     def stream_spot_orderbooks(self, market_ids: List):
         req = spot_exchange_rpc_pb.StreamOrderbookRequest(market_ids=market_ids)
-        return self.stubSpotExchange.StreamOrderbook(req)
+        metadata = self.get_cookie(type="exchange")
+        res = self.stubSpotExchange.StreamOrderbook(req, metadata=metadata)
+        self.set_cookie(res,type="exchange")
+        return res
 
-    def stream_spot_orders(
-        self, market_id: str, **kwargs
-    ):
+    def stream_spot_orders(self, market_id: str, **kwargs):
         req = spot_exchange_rpc_pb.StreamOrdersRequest(
-            market_id=market_id, order_side=kwargs.get("order_side"), subaccount_id=kwargs.get("subaccount_id")
-        )
-        return self.stubSpotExchange.StreamOrders(req)
-
-    def stream_spot_trades(
-        self,
-        market_id: str,
-        **kwargs
-    ):
-        req = spot_exchange_rpc_pb.StreamTradesRequest(
             market_id=market_id,
+            order_side=kwargs.get("order_side"),
+            subaccount_id=kwargs.get("subaccount_id"),
+        )
+        metadata = self.get_cookie(type="exchange")
+        res = self.stubSpotExchange.StreamOrders(req, metadata=metadata)
+        self.set_cookie(res,type="exchange")
+        return res
+
+    def stream_spot_trades(self, **kwargs):
+        req = spot_exchange_rpc_pb.StreamTradesRequest(
+            market_id=kwargs.get("market_id"),
+            market_ids=kwargs.get("market_ids"),
             execution_side=kwargs.get("execution_side"),
             direction=kwargs.get("direction"),
             subaccount_id=kwargs.get("subaccount_id"),
+            subaccount_ids=kwargs.get("subaccount_ids"),
             skip=kwargs.get("skip"),
-            limit=kwargs.get("limit")
+            limit=kwargs.get("limit"),
         )
-        return self.stubSpotExchange.StreamTrades(req)
+        metadata = self.get_cookie(type="exchange")
+        res = self.stubSpotExchange.StreamTrades(req, metadata=metadata)
+        self.set_cookie(res,type="exchange")
+        return res
 
     def get_spot_subaccount_orders(self, subaccount_id: str, **kwargs):
         req = spot_exchange_rpc_pb.SubaccountOrdersListRequest(
@@ -400,11 +469,7 @@ class Client:
         )
         return self.stubSpotExchange.SubaccountOrdersList(req)
 
-    def get_spot_subaccount_trades(
-        self,
-        subaccount_id: str,
-        **kwargs
-    ):
+    def get_spot_subaccount_trades(self, subaccount_id: str, **kwargs):
         req = spot_exchange_rpc_pb.SubaccountTradesListRequest(
             subaccount_id=subaccount_id,
             market_id=kwargs.get("market_id"),
@@ -421,53 +486,86 @@ class Client:
 
     def get_derivative_markets(self, **kwargs):
         req = derivative_exchange_rpc_pb.MarketsRequest(
-            market_status=kwargs.get("market_status"), quote_denom=kwargs.get("quote_denom")
+            market_status=kwargs.get("market_status"),
+            quote_denom=kwargs.get("quote_denom"),
         )
         return self.stubDerivativeExchange.Markets(req)
 
-    def stream_derivative_markets(self):
-        req = derivative_exchange_rpc_pb.StreamMarketRequest()
-        return self.stubDerivativeExchange.StreamMarket(req)
+    def stream_derivative_markets(self, **kwargs):
+        req = derivative_exchange_rpc_pb.StreamMarketRequest(
+            market_ids=kwargs.get("market_ids"))
+        metadata = self.get_cookie(type="exchange")
+        res = self.stubDerivativeExchange.StreamMarket(req, metadata=metadata)
+        self.set_cookie(res,type="exchange")
+        return res
 
     def get_derivative_orderbook(self, market_id: str):
         req = derivative_exchange_rpc_pb.OrderbookRequest(market_id=market_id)
         return self.stubDerivativeExchange.Orderbook(req)
 
-    def get_derivative_orders(
-        self, market_id: str, **kwargs
-    ):
+    def get_derivative_orderbooks(self, market_ids: List):
+        req = derivative_exchange_rpc_pb.OrderbooksRequest(market_ids=market_ids)
+        return self.stubDerivativeExchange.Orderbooks(req)
+
+    def get_derivative_orders(self, market_id: str, **kwargs):
         req = derivative_exchange_rpc_pb.OrdersRequest(
-            market_id=market_id, order_side=kwargs.get("order_side"), subaccount_id=kwargs.get("subaccount_id")
+            market_id=market_id,
+            order_side=kwargs.get("order_side"),
+            subaccount_id=kwargs.get("subaccount_id"),
         )
         return self.stubDerivativeExchange.Orders(req)
 
     def get_derivative_trades(self, market_id: str, **kwargs):
         req = derivative_exchange_rpc_pb.TradesRequest(
-            market_id=market_id, subaccount_id=kwargs.get("subaccount_id"), execution_side=kwargs.get("execution_side"), direction=kwargs.get("direction"), skip=kwargs.get("skip"), limit=kwargs.get("limit")
+            market_id=market_id,
+            subaccount_id=kwargs.get("subaccount_id"),
+            execution_side=kwargs.get("execution_side"),
+            direction=kwargs.get("direction"),
+            skip=kwargs.get("skip"),
+            limit=kwargs.get("limit"),
         )
         return self.stubDerivativeExchange.Trades(req)
 
     def stream_derivative_orderbook(self, market_id: str):
         req = derivative_exchange_rpc_pb.StreamOrderbookRequest(market_ids=[market_id])
-        return self.stubDerivativeExchange.StreamOrderbook(req)
+        metadata = self.get_cookie(type="exchange")
+        res = self.stubDerivativeExchange.StreamOrderbook(req, metadata=metadata)
+        self.set_cookie(res,type="exchange")
+        return res
 
     def stream_derivative_orderbooks(self, market_ids: List):
         req = derivative_exchange_rpc_pb.StreamOrderbookRequest(market_ids=market_ids)
-        return self.stubDerivativeExchange.StreamOrderbook(req)
+        metadata = self.get_cookie(type="exchange")
+        res = self.stubDerivativeExchange.StreamOrderbook(req, metadata=metadata)
+        self.set_cookie(res,type="exchange")
+        return res
 
-    def stream_derivative_orders(
-        self, market_id: str, **kwargs
-    ):
+    def stream_derivative_orders(self, market_id: str, **kwargs):
         req = derivative_exchange_rpc_pb.StreamOrdersRequest(
-            market_id=market_id, order_side=kwargs.get("order_side"), subaccount_id=kwargs.get("subaccount_id")
+            market_id=market_id,
+            order_side=kwargs.get("order_side"),
+            subaccount_id=kwargs.get("subaccount_id"),
         )
-        return self.stubDerivativeExchange.StreamOrders(req)
+        metadata = self.get_cookie(type="exchange")
+        res = self.stubDerivativeExchange.StreamOrders(req, metadata=metadata)
+        self.set_cookie(res,type="exchange")
+        return res
 
-    def stream_derivative_trades(self, market_id: str, **kwargs):
+    def stream_derivative_trades(self, **kwargs):
         req = derivative_exchange_rpc_pb.StreamTradesRequest(
-            market_id=market_id, subaccount_id=kwargs.get("subaccount_id"), execution_side=kwargs.get("execution_side"), direction=kwargs.get("direction"), skip=kwargs.get("skip"), limit=kwargs.get("limit")
+            market_id=kwargs.get("market_id"),
+            market_ids=kwargs.get("market_ids"),
+            subaccount_id=kwargs.get("subaccount_id"),
+            subaccount_ids=kwargs.get("subaccount_ids"),
+            execution_side=kwargs.get("execution_side"),
+            direction=kwargs.get("direction"),
+            skip=kwargs.get("skip"),
+            limit=kwargs.get("limit"),
         )
-        return self.stubDerivativeExchange.StreamTrades(req)
+        metadata = self.get_cookie(type="exchange")
+        res = self.stubDerivativeExchange.StreamTrades(req, metadata=metadata)
+        self.set_cookie(res,type="exchange")
+        return res
 
     def get_derivative_positions(self, market_id: str, **kwargs):
         req = derivative_exchange_rpc_pb.PositionsRequest(
@@ -475,14 +573,22 @@ class Client:
         )
         return self.stubDerivativeExchange.Positions(req)
 
-    def stream_derivative_positions(self, market_id: str, **kwargs):
+    def stream_derivative_positions(self, **kwargs):
         req = derivative_exchange_rpc_pb.StreamPositionsRequest(
-            market_id=market_id, subaccount_id=kwargs.get("subaccount_id")
+            market_id=kwargs.get("market_id"),
+            market_ids=kwargs.get("market_ids"),
+            subaccount_id=kwargs.get("subaccount_id"),
+            subaccount_ids=kwargs.get("subaccount_ids")
         )
-        return self.stubDerivativeExchange.StreamPositions(req)
+        metadata = self.get_cookie(type="exchange")
+        res = self.stubDerivativeExchange.StreamPositions(req, metadata=metadata)
+        self.set_cookie(res,type="exchange")
+        return res
 
     def get_derivative_liquidable_positions(self, **kwargs):
-        req = derivative_exchange_rpc_pb.LiquidablePositionsRequest(market_id=kwargs.get("market_id"))
+        req = derivative_exchange_rpc_pb.LiquidablePositionsRequest(
+            market_id=kwargs.get("market_id")
+        )
         return self.stubDerivativeExchange.LiquidablePositions(req)
 
     def get_derivative_subaccount_orders(self, subaccount_id: str, **kwargs):
@@ -491,11 +597,7 @@ class Client:
         )
         return self.stubDerivativeExchange.SubaccountOrdersList(req)
 
-    def get_derivative_subaccount_trades(
-        self,
-        subaccount_id: str,
-        **kwargs
-    ):
+    def get_derivative_subaccount_trades(self, subaccount_id: str, **kwargs):
         req = derivative_exchange_rpc_pb.SubaccountTradesListRequest(
             subaccount_id=subaccount_id,
             market_id=kwargs.get("market_id"),
@@ -506,6 +608,17 @@ class Client:
 
     def get_funding_payments(self, subaccount_id: str, **kwargs):
         req = derivative_exchange_rpc_pb.FundingPaymentsRequest(
-            subaccount_id=subaccount_id, market_id=kwargs.get("market_id"), skip=kwargs.get("skip"), limit=kwargs.get("limit")
+            subaccount_id=subaccount_id,
+            market_id=kwargs.get("market_id"),
+            skip=kwargs.get("skip"),
+            limit=kwargs.get("limit"),
         )
         return self.stubDerivativeExchange.FundingPayments(req)
+
+    def get_funding_rates(self, market_id: str, **kwargs):
+        req = derivative_exchange_rpc_pb.FundingRatesRequest(
+            market_id=market_id,
+            skip=kwargs.get("skip"),
+            limit=kwargs.get("limit"),
+        )
+        return self.stubDerivativeExchange.FundingRates(req)
